@@ -16,8 +16,6 @@
 Utilities for dealing with paths required during model training and inference.
 """
 
-from typing import Optional
-
 import pathlib
 import shutil
 
@@ -32,8 +30,8 @@ def clear_training() -> None:
     r"""
     Removes checkpoints and tensorboard logs.
     """
-    shutil.rmtree(CHECKPOINT_DIR)
-    shutil.rmtree(TENSORBOARD_DIR)
+    shutil.rmtree(CHECKPOINT_DIR, ignore_errors=True)
+    shutil.rmtree(TENSORBOARD_DIR, ignore_errors=True)
 
 
 def get_board_size_str(h: int, w: int) -> str:
