@@ -34,6 +34,7 @@ from torch.utils.tensorboard import SummaryWriter
 import tqdm
 from tqdm.contrib.logging import tqdm_logging_redirect
 
+import slidingpuzzle.nn.models as models
 import slidingpuzzle.nn.paths as paths
 from slidingpuzzle.nn.dataset import SlidingPuzzleDataset
 from slidingpuzzle.nn.eval import accuracy, evaluate
@@ -69,6 +70,7 @@ def load_checkpoint(
     model: nn.Module,
     tag: str = "acc",
     optimizer: Optional[optim.Optimizer] = None,
+    path: Optional[str] = None,
 ) -> dict:
     """
     Loads a model and optimizer state from the specified epoch.
@@ -86,15 +88,17 @@ def load_checkpoint(
     Args:
         model: The module to load weights into
         optimizer: The optimizer state to restore. If ``None``, ignored.
-        tag: The checkpoint tag to load. If ``None``, latest checkpo
+        tag: The checkpoint tag to load. If ``None``, latest checkpoint is used
+        path: Path to checkpoint. If None, is default path is determined from tag
 
     Returns:
         The checkpoint ``dict``. If not found, returns ``dict`` with default values
         that can be used at the start of search.
     """
-    checkpoint_path = paths.get_checkpoint_path(model.h, model.w, tag)
+    if path is None:
+        path = paths.get_checkpoint_path(model.h, model.w, tag)
     try:
-        checkpoint = torch.load(str(checkpoint_path))
+        checkpoint = torch.load(str(path))
         model.load_state_dict(checkpoint["model_state_dict"])
         if optimizer is not None:
             optimizer.load_state_dict(checkpoint["optimizer_state_dict"])
@@ -166,16 +170,16 @@ def linear_regression_beta(data):
 def train(
     model: nn.Module,
     dataset: SlidingPuzzleDataset,
-    train_fraction: float = 0.95,
+    train_fraction: float = 0.9,
+    lr=0.0001,
     num_epochs: int = 0,
     batch_size: int = 256,
-    early_quit_epochs: int = 256,
-    early_quit_threshold: float = 1e-5,
+    early_quit_epochs: int = 100,
+    early_quit_beta: float = 1e-4,
     device: Optional[str] = None,
     checkpoint_freq: int = 50,
     tensorboard_dir: str = paths.TENSORBOARD_DIR,
     seed: int = 0,
-    **kwargs,
 ) -> None:
     """
     Trains a model for ``num_epochs``. If no prior model is found, a new one is
@@ -185,35 +189,44 @@ def train(
     so that training may be paused and resumed without leaking test data.
 
     Note:
-        If you change your dataset between runs, the train/test split will no longer
-        be consistent with original training, so training will need to be restarted
-        to prevent leaking test data.
+        If you change your dataset or seed between runs, the train/test split will
+        no longer be consistent with original training, so training will need to be
+        restarted (or other measures taken) to prevent leaking test data.
 
     Args:
         model: The model instance to train
         dataset: A dataset to use.
         train_fraction: The train/test split
+        lr: The learning rate for the optimizer
         num_epochs: Total number of epochs model should be trained for. Use 0 to run
             until the ``early_quit_epochs`` logic is hit.
         batch_size: Batch size to use for training
         early_quit_epochs: We will hold an ``early_quit_epochs``-sized window of test
             loss values. If the linear regression slope of these data points is
-            > ``early_quit_threshold``, we will terminate training early. Use 0 to
-            disable this feature and always run until ``num_epochs`` of training have
+            > ``early_quit_beta``, we will terminate training early. Use 0 to disable
+            this feature and always run until ``num_epochs`` of training have
             completed. If both this and ``num_epochs`` are 0, training will run until
             manually interrupted.
         early_quit_beta: If the slope of the test loss rises above this value, training
             is terminated.
         device: Device to train on. Default will autodetect CUDA or use CPU
         checkpoint_freq: Model will be checkpointed each time this many epochs
-            elapse. If 0, no epoch checkpointint will be used. (Highest test acc. will
+            elapse. If 0, no epoch checkpoints will be used. (Highest test acc. will
             still be checkpointed and also final checkpoint on termination.)
         tensorboard_dir: The root tensorboard dir for logs. Default is "tensorboard".
         seed: Seed used for torch random utilities for reproducibility
+
+    Note:
+        Every time a new test/acc high is observed, the early_quit_window will be
+        cleared so that training may continue for longer.
     """
-    set_seed(seed)
     h, w = model.w, model.h
 
+    if device is None:
+        device = models.DEVICE
+
+    set_seed(seed)
+
     # prepare tensorboard to record training
     url = launch_tensorboard(tensorboard_dir)
     log.info(f"Tensorboard launched: {url}")
@@ -235,12 +248,11 @@ def train(
     train_dataset, test_dataset = torch.utils.data.random_split(
         dataset, [train_size, test_size], generator=torch.Generator().manual_seed(seed)
     )
+    log.info(f"train_size={train_size}, test_size={test_size}")
 
     # prepare model
-    if device is None:
-        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
     model.to(device)
-    optimizer = optim.SGD(model.parameters(), lr=0.0001, momentum=0.9)
+    optimizer = optim.SGD(model.parameters(), lr=lr, momentum=0.9)
     highest_acc = load_checkpoint(model, tag="acc", optimizer=optimizer)[
         "test_accuracy"
     ]
@@ -300,6 +312,7 @@ def train(
             if test_accuracy > highest_acc:
                 # save a tagged checkpoint for highest acc
                 highest_acc = test_accuracy
+                test_loss_window.clear()
                 state = get_state_dict(
                     model, optimizer, epoch=epoch, test_accuracy=test_accuracy
                 )
@@ -312,12 +325,9 @@ def train(
                 save_checkpoint(state, f"epoch_{epoch}")
             if early_quit_epochs > 0 and len(test_loss_window) == early_quit_epochs:
                 # if we are using early quitting, check the overall trendline is down
-                if linear_regression_beta(test_loss_window) > early_quit_threshold:
+                if linear_regression_beta(test_loss_window) > early_quit_beta:
                     with tqdm_logging_redirect():
-                        log.info(
-                            f"Early quit threshold reached at epoch {epoch}, "
-                            f"highest_acc={highest_acc}"
-                        )
+                        log.info(f"Early quit threshold reached at epoch {epoch}.")
                     break
             if num_epochs and epoch == num_epochs - 1:
                 with tqdm_logging_redirect():
@@ -330,6 +340,7 @@ def train(
     finally:
         pbar.close()
 
+    log.info(f"highest_acc={round(highest_acc, 4)}")
     # save final state in case we were interrupted
     state = get_state_dict(model, optimizer, epoch=epoch, test_accuracy=test_accuracy)
     save_checkpoint(state, "latest")
